﻿using System;
using System.Text.RegularExpressions;
using SDRSharp.Radio;
using NDde.Client;

namespace SDRSharp.DDETracker
{
    public delegate void AdviseArrivedDelegate(object sender, AdviseArrivedEventArgs args);
    public delegate void ServerDisconnectDelegate(object sender);

    public class WispDdeClient
    {
        private string _applicationName;       
        private string _linkTopic;
        private string _linkItem;
        private readonly DdeClient _client;     
        public event AdviseArrivedDelegate AdviseArrivedEvent;
        public event ServerDisconnectDelegate ServerDisconnectEvent;

        #region Public Properties

        public string ApplicationName
        {
            get { return _applicationName; }            
        }

        public string LinkTopic
        {
            get { return _linkTopic; }            
        }

        public string LinkItem
        {
            get { return _linkItem; }            
        }
        
        public bool IsConnected
        {
            get { return _client.IsConnected; }
        }

        #endregion

        public WispDdeClient(string appName, string topic, string item)
        {            
            _applicationName = appName;
            _linkTopic = topic;
            _linkItem = item;            
            _client = new DdeClient(_applicationName,_linkItem);
            _client.Advise += client_Advise;
            _client.Disconnected += client_Disconnected;
        }
              
        public void Connect()
        {
            if (!_client.IsConnected)
            {
                _client.Connect();
                _client.StartAdvise(_linkTopic, 1, true, 60000);
            }
        }

        public void Disconnect()
        {
            if (_client.IsConnected)
            {
                _client.Disconnect();
            }
        }

        private void client_Advise(object sender, DdeAdviseEventArgs args)
        {
            if (string.IsNullOrEmpty(args.Text))
            {
                return;
            }

            var eventargs = new AdviseArrivedEventArgs();
            try
            {
                
                char [] splitter = { ' ' };
                var elements = args.Text.Split(splitter);
                if (elements.Length < 4)
                {
                    return;
                }
         
                var count = 0;
                foreach (string e in elements)
                {
                    if (e.Length < 3)
                    {
                        continue;
                    }
                    var prefix = e.Substring(0, 2);
                    switch (prefix.ToUpper())
                    {
                        case "SN":
                            eventargs.Name = (string)e.Remove(0, 2);
                            count++;
                            break;
                        case "AZ":
                            eventargs.Azimuth = Convert.ToSingle(e.Remove(0, 2));
                            count++;
                            break;
                        case "EL":
                            eventargs.Elevation = Convert.ToSingle(e.Remove(0, 2));
                            count++;
                            break;
                        case "DN":
                            eventargs.DownLinkFrequency = Convert.ToInt64(e.Remove(0, 2));
                            count++;
                            break;
                    }
                    if (count == 4)
                    {
                        break;
                    }
                }
                eventargs.RawData = args.Text;

                if (AdviseArrivedEvent != null)
                {
                    AdviseArrivedEvent(this, eventargs);
                }

            }
            catch (Exception e)
            {
                Console.WriteLine("Exception {0}", e.Message);
            }
        }                                
    

        void client_Disconnected(object sender, DdeDisconnectedEventArgs e)
        {
            if (e.IsServerInitiated)
            {
                if (ServerDisconnectEvent != null)
                {
                    ServerDisconnectEvent(this);
                }
            }
        }        
    }

    public class AdviseArrivedEventArgs:EventArgs
    {
        public string Name;
        public float Azimuth;
        public float Elevation;
        public long DownLinkFrequency;                
        public string RawData;        
    }
}
